------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.8     080922
--
--                      'T a s k s _ I n s p e c t o r'
--
--                                 Body
--
--
--  File 'tasks_inspector.adb'                                        By Mar.
--
--
--  Operantions to send scheduling events to the host using a serial
--  port. The events sent using 'Send_Event' can be interpretated by the
--  "Tasks Inspector" tool to display the tasks activity graphically.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2008, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with MaRTE.Integer_Types; use MaRTE.Integer_Types;
with MaRTE.Debug_Messages;

package body MaRTE.Spy is


   ----------------------------------
   -- Imports from 'marte-spy_c.c' --
   ----------------------------------
   procedure Init_Serial_Port_B115200 (Com_Port : in Int);
   pragma Import (C, Init_Serial_Port_B115200,
                    "_marte_init_serial_port_B115200");
   procedure Send_Event_One_Object (Com_Port  : in Int;
                                    Timestamp : in HAL.HWTime;
                                    C_Op1     : in Character;
                                    C_Op2     : in Character;
                                    Id1       : in Int;
                                    Prio1     : in Int);
   pragma Import (C, Send_Event_One_Object, "_marte_send_event_one_object");
   procedure Send_Event_Two_Objects (Com_Port : in Int;
                                     Timestamp : in HAL.HWTime;
                                     C_Op1     : in Character;
                                     C_Op2     : in Character;
                                     Id1       : in Int;
                                     Prio1     : in Int;
                                     Id2       : in Int;
                                     Prio2     : in Int);
   pragma Import (C, Send_Event_Two_Objects, "_marte_send_event_two_objects");


   ------------------
   -- Initial_Time --
   ------------------
   Initial_Time : HAL.HWTime;

   ----------
   -- Init --
   ----------
   procedure Init (Num_Tasks_Mx     : in Integer;
                   Task_Priority_Mx : in Integer;
                   HWClock_Freq     : in HAL.HWTime) is
      Interrupts_Enabled : Boolean := False;
      Flags : Integer;
   begin
      if MaRTE.Debug_Messages.Tasks_Inspector_Messages then
         if HAL.Are_Interrupts_Enabled then
            MaRTE.Kernel.Enter_Critic_Section (Flags);
            Interrupts_Enabled := True;
         end if;

         Initial_Time := HAL.Get_HWTime_Slow;
         --  Initialice serial port
         Init_Serial_Port_B115200 (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port);
         --  Send Initial data
         Send_Event_One_Object (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port,
                                0, 'I', 'D',
                                Int (Num_Tasks_Mx), Int (Task_Priority_Mx));
         Send_Event_One_Object (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port,
                                0, 'C', 'F',
                                Int (HWClock_Freq), 0);

         if Interrupts_Enabled then
            MaRTE.Kernel.Leave_Critic_Section (Flags);
         end if;
      end if;
   end Init;

   ----------------
   -- Send_Event --
   ----------------
   procedure Send_Event (Timestamp : in HAL.HWTime;
                         Operation : in String;
                         T1        : in MaRTE.Kernel.Task_Id;
                         T2        : in MaRTE.Kernel.Task_Id) is
      use HAL;
      Interrupts_Enabled : Boolean := False;
      Flags : Integer;
   begin
      if MaRTE.Debug_Messages.Tasks_Inspector_Messages then
         if HAL.Are_Interrupts_Enabled then
            MaRTE.Kernel.Enter_Critic_Section (Flags);
            Interrupts_Enabled := True;
         end if;

         if MaRTE.Kernel.Tasks_Lists."=" (T2, null) then
            Send_Event_One_Object (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port,
                                   Timestamp - Initial_Time,
                                   Operation (1), Operation (2),
                                   Int (T1.Id), Int (T1.Active_Prio));
         else
            Send_Event_Two_Objects (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port,
                                    Timestamp - Initial_Time,
                                    Operation (1), Operation (2),
                                    Int (T1.Id), Int (T1.Active_Prio),
                                    Int (T2.Id), Int (T2.Active_Prio));
         end if;

         if Interrupts_Enabled then
            MaRTE.Kernel.Leave_Critic_Section (Flags);
         end if;
      end if;
   end Send_Event;

   ----------------
   -- Send_Event --
   ----------------
   procedure Send_Event (Timestamp : in HAL.HWTime;
                         Operation : in String;
                         Id        : in Integer;
                         Prio      : in MaRTE.Kernel.Task_Priority) is
      use HAL;
      Interrupts_Enabled : Boolean := False;
      Flags : Integer;
   begin
      if MaRTE.Debug_Messages.Tasks_Inspector_Messages then
         if HAL.Are_Interrupts_Enabled then
            MaRTE.Kernel.Enter_Critic_Section (Flags);
            Interrupts_Enabled := True;
         end if;

         Send_Event_One_Object (MaRTE.Debug_Messages.Tasks_Inspector_Serial_Port,
                                Timestamp - Initial_Time,
                                Operation (1), Operation (2),
                                Int (Id), Int (Prio));

         if Interrupts_Enabled then
            MaRTE.Kernel.Leave_Critic_Section (Flags);
         end if;
      end if;
   end Send_Event;
end MaRTE.Spy;
