/*----------------------------------------------------------------------------
 *--------------------        M a R T E     O S        -----------------------
 *----------------------------------------------------------------------------
 *                                                             V1.8     080922
 *
 *                   't a s k s _ i n s p e c t o r _ c'
 *
 *                                   C
 *
 *
 * File 'tasks_inspector_c.c'                                          By Mar.
 *
 * Operantions to send scheduling events to the host using a serial
 * port. The events sent using 'Send_Event' can be interpretated by
 * the "Tasks Inspector" tool to display the tasks activity
 * graphically.
 *
 * ----------------------------------------------------------------------
 *  Copyright (C) 2000-2008, Universidad de Cantabria, SPAIN
 *
 *  MaRTE OS web page: http://marte.unican.es
 *  Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
 *                     Michael Gonzalez Harbour      mgh@unican.es
 *
 * MaRTE OS  is free software; you can  redistribute it and/or  modify it
 * under the terms of the GNU General Public License  as published by the
 * Free Software Foundation;  either  version 2, or (at  your option) any
 * later version.
 *
 * MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
 * WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 * MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 * General Public License for more details.
 *
 * You should have received  a  copy of  the  GNU General Public  License
 * distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 * Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 * 02111-1307, USA.
 *
 * As a  special exception, if you  link this  unit  with other  files to
 * produce an   executable,   this unit  does  not  by  itself cause  the
 * resulting executable to be covered by the  GNU General Public License.
 * This exception does  not however invalidate  any other reasons why the
 * executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <oskit/x86/pc/com_cons.h>
#include <oskit/tty.h>
#undef _POSIX_SOURCE // this symbol is defined when compiling for
		     // linux_lib_arch. It should be undefined.
#include <oskit/c/termios.h>

#define BUFF_LENGTH 50
char buff[BUFF_LENGTH]; // can be declared here because the send event
 // functions are going to be called from critic regions (no
 // preemption).


/*
 * send_event_one_object
 */
void send_event_one_object (int port,
			    unsigned long long timestamp,
			    char c_op1, // I don't want to pass strings
			    char c_op2,
			    int id, int prio) 
{
  char *p = buff;
  snprintf (buff, BUFF_LENGTH, "\n#%1qu#%c%c(%1d,%1d)#", 
	    timestamp, c_op1, c_op2, id, prio);

  do {
    com_cons_putchar(port, (int) *p);
  } while (*(++p));  
}

/*
 * send_event_two_objects
 */
void send_event_two_objects (int port,
			     unsigned long long timestamp,
			     char c_op1, // I don't want to pass strings
			     char c_op2,
			     int id1, int prio1,
			     int id2, int prio2) 
{
  char *p = buff;
  snprintf (buff, BUFF_LENGTH, "\n#%1qu#%c%c(%1d,%1d)->(%1d,%1d)#", 
	    timestamp, c_op1, c_op2, 
	    id1, prio1, id2, prio2);

  do {
    com_cons_putchar(port, (int) *p);
  } while (*(++p));  
}


/*
 * Init Serial Port
 */
void init_serial_port_B115200 (int port)
{
  base_cooked_termios.c_ispeed = B115200;
  base_cooked_termios.c_ospeed = B115200;
  com_cons_init(port, &base_cooked_termios);
}
		   
