------------------------------------------------------------------------------
--                                                                          --
--                GNU ADA RUN-TIME LIBRARY (GNARL) COMPONENTS               --
--                                                                          --
--                   S Y S T E M . O S _ I N T E R F A C E                  --
--                                                                          --
--                                  S p e c                                 --
--                                                                          --
--             Copyright (C) 1991-1994, Florida State University            --
--             Copyright (C) 1995-2004, Free Software Foundation, Inc.      --
--                                                                          --
-- GNARL is free software; you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. GNARL is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNARL; see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University.       --
-- Extensive contributions were provided by Ada Core Technologies, Inc.     --
--                                                                          --
------------------------------------------------------------------------------

--  This package encapsulates all direct interfaces to OS services
--  that are needed by children of System.

------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.8     080922
--
--  This is a MaRTE OS version of this package. Based in the original
--  GNU/Linux (GNU/LinuxThreads) version of this package.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2008, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------


with Interfaces.C;

--  << MaRTE OS
with Pthread;
with Kernel.Tasks_Operations.Attributes;
with Time;
with Signal;
with Sched;
with General_Constants;
with Error_Codes;
with Signals_Values;
--  MaRTE OS >>

with Unchecked_Conversion;

package System.OS_Interface is
   --  << MaRTE OS
   --  pragma Preelaborate;
   pragma Elaborate_Body;
   --  MaRTE OS >>

   --  << MaRTE OS
   --  pragma Linker_Options ("-lpthread");
   --  MaRTE OS >>

   --  << MaRTE OS
   package PSignal renames Signal;
   --  To avoid confusion with type 'Signal'
   --  MaRTE OS >>

   subtype int            is Interfaces.C.int;
   subtype char           is Interfaces.C.char;
   subtype short          is Interfaces.C.short;
   subtype long           is Interfaces.C.long;
   subtype unsigned       is Interfaces.C.unsigned;
   subtype unsigned_short is Interfaces.C.unsigned_short;
   subtype unsigned_long  is Interfaces.C.unsigned_long;
   subtype unsigned_char  is Interfaces.C.unsigned_char;
   subtype plain_char     is Interfaces.C.plain_char;
   subtype size_t         is Interfaces.C.size_t;

   -----------
   -- Errno --
   -----------

   function errno return int;
   --  << MaRTE OS
   --  EAGAIN    : constant := 11;
   --  EINTR     : constant := 4;
   --  EINVAL    : constant := 22;
   --  ENOMEM    : constant := 12;
   --  EPERM     : constant := 1;
   --  ETIMEDOUT : constant := 110;
   EAGAIN    : constant := Error_Codes.RESOURCE_TEMPORARILY_UNAVAILABLE;
   EINTR     : constant := Error_Codes.INTERRUPTED_OPERATION;
   EINVAL    : constant := Error_Codes.INVALID_ARGUMENT;
   ENOMEM    : constant := Error_Codes.NOT_ENOUGH_SPACE;
   EPERM     : constant := Error_Codes.OPERATION_NOT_PERMITTED;
   ETIMEDOUT : constant := Error_Codes.TIMED_OUT;
   --  MaRTE OS >>

   -------------
   -- Signals --
   -------------

   --  << MaRTE OS
--     Max_Interrupt : constant := 63;
--     type Signal is new int range 0 .. Max_Interrupt;
--     for Signal'Size use int'Size;

--     SIGHUP     : constant := 1; --  hangup
--     SIGINT     : constant := 2; --  interrupt (rubout)
--     SIGQUIT    : constant := 3; --  quit (ASCD FS)
--     SIGILL     : constant := 4; --  illegal instruction (not reset)
--     SIGTRAP    : constant := 5; --  trace trap (not reset)
--     SIGIOT     : constant := 6; --  IOT instruction
--  SIGABRT    : constant := 6; --  used by abort, replace SIGIOT in the future
--     SIGFPE     : constant := 8; --  floating point exception
--     SIGKILL    : constant := 9; --  kill (cannot be caught or ignored)
--     SIGBUS     : constant := 7; --  bus error
--     SIGSEGV    : constant := 11; --  segmentation violation
--     SIGPIPE    : constant := 13; --  write on a pipe with no one to read it
--     SIGALRM    : constant := 14; --  alarm clock
--     SIGTERM    : constant := 15; --  software termination signal from kill
--     SIGUSR1    : constant := 10; --  user defined signal 1
--     SIGUSR2    : constant := 12; --  user defined signal 2
--     SIGCLD     : constant := 17; --  alias for SIGCHLD
--     SIGCHLD    : constant := 17; --  child status change
--     SIGPWR     : constant := 30; --  power-fail restart
--     SIGWINCH   : constant := 28; --  window size change
--     SIGURG     : constant := 23; --  urgent condition on IO channel
--     SIGPOLL    : constant := 29; --  pollable event occurred
--     SIGIO      : constant := 29; --  I/O now possible (4.2 BSD)
--     SIGLOST    : constant := 29; --  File lock lost
--     SIGSTOP    : constant := 19; --  stop (cannot be caught or ignored)
--     SIGTSTP    : constant := 20; --  user stop requested from tty
--     SIGCONT    : constant := 18; --  stopped process has been continued
--     SIGTTIN    : constant := 21; --  background tty read attempted
--     SIGTTOU    : constant := 22; --  background tty write attempted
--     SIGVTALRM  : constant := 26; --  virtual timer expired
--     SIGPROF    : constant := 27; --  profiling timer expired
--     SIGXCPU    : constant := 24; --  CPU time limit exceeded
--     SIGXFSZ    : constant := 25; --  filesize limit exceeded
--     SIGUNUSED  : constant := 31; --  unused signal (GNU/Linux)
--     SIGSTKFLT  : constant := 16; --  coprocessor stack fault (Linux)
--     SIGLTHRRES : constant := 32; --  GNU/LinuxThreads restart signal
--     SIGLTHRCAN : constant := 33; --  GNU/LinuxThreads cancel signal
--     SIGLTHRDBG : constant := 34; --  GNU/LinuxThreads debugger signal

   Max_Interrupt : constant := PSignal.Signal'Last;
   subtype Signal is PSignal.Signal;

   SIGHUP     : constant :=
     Signals_Values.SIGHUP; --  hangup
   SIGINT     : constant := -- 2
     Signals_Values.SIGINT; --  interrupt (rubout)
   SIGQUIT    : constant := -- 3
     Signals_Values.SIGQUIT; --  quit (ASCD FS)
   SIGILL     : constant := -- 4
     Signals_Values.SIGILL; --  illegal instruction (not reset)
   SIGTRAP    : constant := -- 5
     Signals_Values.SIGTRAP; --  trace trap (not reset)
   --  SIGIOT     : constant := -- 6
   --    Signals_Values.SIGIOT; --  IOT instruction
   SIGABRT    : constant := -- 6
     Signals_Values.SIGABRT; --  used by abort, replace SIGIOT in the  future
   SIGFPE     : constant := -- 8
     Signals_Values.SIGFPE; --  floating point exception
   SIGKILL    : constant := -- 9
     Signals_Values.SIGKILL; --  kill (cannot be caught or ignored)
   SIGBUS     : constant := -- 7
     Signals_Values.SIGBUS; --  bus error
   SIGSEGV    : constant := -- 11
     Signals_Values.SIGSEGV; --  segmentation violation
   SIGPIPE    : constant := -- 13
     Signals_Values.SIGPIPE; --  write on a pipe with no one to read it
   SIGALRM    : constant := -- 14
     Signals_Values.SIGALRM; --  alarm clock
   SIGTERM    : constant := -- 15
     Signals_Values.SIGTERM; --  software termination signal from kill
   SIGUSR1    : constant := -- 10
     Signals_Values.SIGUSR1; --  user defined signal 1
   SIGUSR2    : constant := -- 12
     Signals_Values.SIGUSR2; --  user defined signal 2
   --  SIGCLD     : constant := -- 17
   --    Signals_Values.SIGCLD; --  alias for SIGCHLD
   SIGCHLD    : constant := -- 17
     Signals_Values.SIGCHLD; --  child status change
   --  SIGPWR     : constant := -- 30
   --    Signals_Values.SIGPWR; --  power-fail restart
   --  SIGWINCH   : constant := -- 28
   --    Signals_Values.SIGWINCH; --  window size change
   --  SIGURG     : constant := -- 23
   --    Signals_Values.SIGURG; --  urgent condition on IO channel
   --  SIGPOLL    : constant := -- 29
   --    Signals_Values.SIGPOLL; --  pollable event occurred
   --  SIGIO      : constant := -- 29
   --    Signals_Values.SIGIO; --  I/O now possible (4.2 BSD)
   --  SIGLOST    : constant := -- 29
   --    Signals_Values.SIGLOST; --  File lock lost
   SIGSTOP    : constant := -- 19
     Signals_Values.SIGSTOP; --  stop (cannot be caught or ignored)
   SIGTSTP    : constant := -- 20
     Signals_Values.SIGTSTP; --  user stop requested from tty
   SIGCONT    : constant := -- 18
     Signals_Values.SIGCONT; --  stopped process has been continued
   SIGTTIN    : constant := -- 21
     Signals_Values.SIGTTIN; --  background tty read attempted
   SIGTTOU    : constant := -- 22
     Signals_Values.SIGTTOU; --  background tty write attempted
   SIGVTALRM  : constant := -- 26
     Signals_Values.SIGVTALRM; --  virtual timer expired
   --  SIGPROF    : constant := -- 27
   --    Signals_Values.SIGPROF; --  profiling timer expired
   --  SIGXCPU    : constant := -- 24
   --    Signals_Values.SIGXCPU; --  CPU time limit exceeded
   --  SIGXFSZ    : constant := -- 25
   --    Signals_Values.SIGXFSZ; --  filesize limit exceeded
   SIGUNUSED  : constant := -- 31
     Signals_Values.SIGUNUSED;  --  unused signal (Linux)
   --  SIGSTKFLT  : constant := -- 16
   --    Signals_Values.SIGSTKFLT;  --  coprocessor stack fault (Linux)

   --  MaRTE OS >>

   SIGADAABORT : constant := SIGABRT;
   --  Change this if you want to use another signal for task abort.
   --  SIGTERM might be a good one.

   type Signal_Set is array (Natural range <>) of Signal;

   --  << MaRTE OS
   --  Unmasked    : constant Signal_Set := (
--        SIGTRAP,
--       --  To enable debugging on multithreaded applications, mark SIGTRAP to
--        --  be kept unmasked.

--        SIGBUS,

--        SIGTTIN, SIGTTOU, SIGTSTP,
--        --  Keep these three signals unmasked so that background processes
--        --  and IO behaves as normal "C" applications

--        SIGPROF,
--        --  To avoid confusing the profiler

--        SIGKILL, SIGSTOP,
--        --  These two signals actually cannot be masked;
--        --  POSIX simply won't allow it.

--        SIGLTHRRES, SIGLTHRCAN, SIGLTHRDBG);
--        --  These three signals are used by GNU/LinuxThreads starting from
--        --  glibc 2.1 (future 2.2).

   Unmasked    : constant Signal_Set := (SIGTRAP, SIGBUS);
   --  MaRTE OS >>

   Reserved    : constant Signal_Set :=
   --  I am not sure why the following two signals are reserved.
   --  I guess they are not supported by this version of GNU/Linux.
     (SIGVTALRM, SIGUNUSED);

   type sigset_t is private;

   function sigaddset (set : access sigset_t; sig : Signal) return int;
   --  pragma Import (C, sigaddset, "sigaddset"); <-- MaRTE OS

   function sigdelset (set : access sigset_t; sig : Signal) return int;
   --  pragma Import (C, sigdelset, "sigdelset"); <-- MaRTE OS

   function sigfillset (set : access sigset_t) return int;
   --  pragma Import (C, sigfillset, "sigfillset"); <-- MaRTE OS

   function sigismember (set : access sigset_t; sig : Signal) return int;
   --  pragma Import (C, sigismember, "sigismember"); <-- MaRTE OS

   function sigemptyset (set : access sigset_t) return int;
   --  pragma Import (C, sigemptyset, "sigemptyset"); <-- MaRTE OS

   --  << MaRTE OS
   --  type union_type_3 is new String (1 .. 116);
   --  type siginfo_t is record
   --     si_signo : int;
   --     si_code  : int;
   --     si_errno : int;
   --     X_data   : union_type_3;
   --  end record;
   --  pragma Convention (C, siginfo_t);
   --  MaRTE OS >>

   type struct_sigaction is record
      sa_handler   : System.Address;
      sa_mask      : sigset_t;
      --  << MaRTE OS
      --  sa_flags     : unsigned_long;
      --  sa_restorer  : System.Address;
      sa_flags     : int;
      --  MaRTE OS >>
   end record;
   pragma Convention (C, struct_sigaction);
   type struct_sigaction_ptr is access all struct_sigaction;

   type Machine_State is record
      eip : unsigned_long;
      ebx : unsigned_long;
      esp : unsigned_long;
      ebp : unsigned_long;
      esi : unsigned_long;
      edi : unsigned_long;
   end record;
   type Machine_State_Ptr is access all Machine_State;

   --  << MaRTE OS
   --  SA_SIGINFO  : constant := 16#04#;

   --  SIG_BLOCK   : constant := 0;
   --  SIG_UNBLOCK : constant := 1;
   --  SIG_SETMASK : constant := 2;
   SA_SIGINFO  : constant int := General_Constants.SA_SIGINFO;
   SIG_BLOCK   : constant int := int (PSignal.SIG_BLOCK);
   SIG_UNBLOCK : constant int := int (PSignal.SIG_UNBLOCK);
   SIG_SETMASK : constant int := int (PSignal.SIG_SETMASK);
   --  MaRTE OS >>

   --  << MaRTE OS
   --  SIG_DFL : constant := 0;
   --  SIG_IGN : constant := 1;
   SIG_DFL : constant int := int (PSignal.SIG_DFL);
   SIG_IGN : constant int := int (PSignal.SIG_IGN);
   --  MaRTE OS >>

   function sigaction
     (sig  : Signal;
      act  : struct_sigaction_ptr;
      oact : struct_sigaction_ptr) return int;
   --  pragma Import (C, sigaction, "sigaction"); <-- MaRTE OS

   ----------
   -- Time --
   ----------

   --  << MaRTE OS
   --  type timespec is private;
   subtype timespec is Duration;
   --  MaRTE OS>>

   function To_Duration (TS : timespec) return Duration;
   pragma Inline (To_Duration);

   function To_Timespec (D : Duration) return timespec;
   pragma Inline (To_Timespec);

   --  << MaRTE OS
   --  type struct_timeval is private;
   subtype struct_timeval is Time.Timeval;
   --  MaRTE OS >>

   --  << MaRTE OS
   --  function To_Duration (TV : struct_timeval) return Duration;
   --  pragma Inline (To_Duration);
   --  function To_Timeval (D : Duration) return struct_timeval;
   --  pragma Inline (To_Timeval);
   function To_Duration (TV : struct_timeval) return Duration
     renames Time.To_Duration;

   function To_Timeval (D : Duration) return struct_timeval
     renames Time.To_Timeval;
   --  MaRTE OS >>

   function gettimeofday
     (tv : access struct_timeval;
      tz : System.Address := System.Null_Address) return int;
   --  pragma Import (C, gettimeofday, "gettimeofday"); <--- MaRTE OS

   --  function sysconf (name : int) return long;
   --  pragma Import (C, sysconf); <--- MaRTE OS

   --  SC_CLK_TCK : constant := 2; <--- MaRTE OS

   -------------------------
   -- Priority Scheduling --
   -------------------------

   --  << MaRTE OS
   --  SCHED_OTHER : constant := 0;
   --  SCHED_FIFO  : constant := 1;
   --  SCHED_RR    : constant := 2;
   SCHED_FIFO  : constant := General_Constants.SCHED_FIFO;
   SCHED_RR    : constant := General_Constants.SCHED_RR;
   SCHED_OTHER : constant := General_Constants.SCHED_OTHER;
   --  MaRTE OS >>

   -------------
   -- Process --
   -------------

   --  << MaRTE OS
   --  type pid_t is private;
   subtype pid_t is int;
   --  MaRTE OS >>

   --  << MaRTE OS
   --  function kill (pid : pid_t; sig : Signal) return int;
   --  pragma Import (C, kill, "kill");
   function kill (pid : pid_t; sig : Signal) return int renames PSignal.Kill;
   --  MaRTE OS >>

   function getpid return pid_t;
   --  pragma Import (C, getpid, "getpid"); <-- MaRTE OS

   -------------
   -- Threads --
   -------------

   type Thread_Body is access
     function (arg : System.Address) return System.Address;

   function Thread_Body_Access is new
     Unchecked_Conversion (System.Address, Thread_Body);

   --  << MaRTE OS
   --  type pthread_t is new unsigned_long;
   subtype pthread_t is Pthread.Pthread_T;
   subtype Thread_Id        is pthread_t;
   --  MaRTE OS >>

   function To_pthread_t is new Unchecked_Conversion
     (unsigned_long, pthread_t);

   type pthread_mutex_t     is limited private;
   --  << MaRTE OS
   --  type pthread_cond_t      is limited private;
   --  type pthread_attr_t      is limited private;
   --  type pthread_mutexattr_t is limited private;
   --  type pthread_condattr_t  is limited private;
   --  type pthread_key_t       is private;
   subtype pthread_cond_t      is Pthread.Pthread_Cond_T;
   subtype pthread_attr_t      is Pthread.Pthread_Attr_T;
   subtype pthread_mutexattr_t is Pthread.Pthread_Mutexattr_T;
   subtype pthread_condattr_t  is Pthread.Pthread_Condattr_T;
   subtype pthread_key_t       is Pthread.Pthread_Key_T;

   Self_Key : pthread_key_t;
   --  MaRTE OS >>

   --  << MaRTE OS
   --  PTHREAD_CREATE_DETACHED : constant := 1;
   PTHREAD_CREATE_DETACHED : constant :=
     General_Constants.PTHREAD_CREATE_DETACHED;
   --  MaRTE OS >>

   -----------
   -- Stack --
   -----------

   function Get_Stack_Base (thread : pthread_t) return Address;
   pragma Inline (Get_Stack_Base);
   --  This is a dummy procedure to share some GNULLI files

   ---------------------------------------
   -- Nonstandard Thread Initialization --
   ---------------------------------------

   procedure pthread_init;
   pragma Inline (pthread_init);
   --  This is a dummy procedure to share some GNULLI files

   -------------------------
   -- POSIX.1c  Section 3 --
   -------------------------

   function sigwait (set : access sigset_t; sig : access Signal) return int;
   --  pragma Import (C, sigwait, "sigwait"); <-- MaRTE OS

   function pthread_kill (thread : pthread_t; sig : Signal) return int;
   --  pragma Import (C, pthread_kill, "pthread_kill"); <-- MaRTE OS

   type sigset_t_ptr is access all sigset_t;

   function pthread_sigmask
     (how  : int;
      set  : sigset_t_ptr;
      oset : sigset_t_ptr) return int;
   --  pragma Import (C, pthread_sigmask, "pthread_sigmask"); <-- MaRTE OS

   --------------------------
   -- POSIX.1c  Section 11 --
   --------------------------

   --  << MaRTE OS
   function pthread_mutexattr_init
     (attr : access pthread_mutexattr_t) return int
     renames Pthread.Pthread_Mutexattr_Init;
   --  function pthread_mutexattr_init
   --    (attr : access pthread_mutexattr_t) return int;
   --  pragma Import (C, pthread_mutexattr_init, "pthread_mutexattr_init");
   --  MaRTE OS >>

   --  << MaRTE OS
   function pthread_mutexattr_destroy
     (attr : access pthread_mutexattr_t) return int
     renames Pthread.Pthread_Mutexattr_Destroy;
   --  function pthread_mutexattr_destroy
   --    (attr : access pthread_mutexattr_t) return int;
   --  pragma Import (C, pthread_mutexattr_destroy,
   --                 "pthread_mutexattr_destroy");
   --  MaRTE OS >>

   function pthread_mutex_init
     (mutex : access pthread_mutex_t;
      attr  : access pthread_mutexattr_t) return int;
   --  pragma Import (C, pthread_mutex_init,
   --                 "pthread_mutex_init"); <-- MaRTE OS

   function pthread_mutex_destroy (mutex : access pthread_mutex_t) return int;
   --  pragma Import (C, pthread_mutex_destroy,
   --                 "pthread_mutex_destroy"); <-- MaRTE OS

   function pthread_mutex_lock (mutex : access pthread_mutex_t) return int;
   --  pragma Import (C, pthread_mutex_lock,
   --               "pthread_mutex_lock"); <-- MaRTE OS

   function pthread_mutex_unlock (mutex : access pthread_mutex_t) return int;
   --  pragma Import (C, pthread_mutex_unlock,
   --                 "pthread_mutex_unlock"); <-- MaRTE OS

   --  << MaRTE OS
   function pthread_condattr_init
     (attr : access pthread_condattr_t) return int
     renames Pthread.Pthread_Condattr_Init;
   --  function pthread_condattr_init
   --    (attr : access pthread_condattr_t) return int;
   --  pragma Import (C, pthread_condattr_init, "pthread_condattr_init");
   --  MaRTE OS >>

   --  << MaRTE OS
   function pthread_condattr_destroy
     (attr : access pthread_condattr_t) return int
     renames Pthread.Pthread_Condattr_Destroy;
   --  pragma Import (C, pthread_condattr_destroy,
   --                 "pthread_condattr_destroy");
   --  MaRTE OS >>

   --  << MaRTE OS
   function pthread_cond_init
     (cond : access pthread_cond_t;
      attr : access pthread_condattr_t) return int
     renames Pthread.Pthread_Cond_Init;
   --  pragma Import (C, pthread_cond_init, "pthread_cond_init");
   --  MaRTE OS >>

   --  << MaRTE OS
   function pthread_cond_destroy (cond : access pthread_cond_t) return int
     renames Pthread.Pthread_Cond_Destroy;
   --  pragma Import (C, pthread_cond_destroy,
   --                 "pthread_cond_destroy");
   --  MaRTE OS >>

   --  << MaRTE OS
   function pthread_cond_signal (cond : access pthread_cond_t) return int
     renames Pthread.Pthread_Cond_Signal;
   --  pragma Import (C, pthread_cond_signal,
   --                 "pthread_cond_signal");
   --  MaRTE OS >>

   function pthread_cond_wait
     (cond  : access pthread_cond_t;
      mutex : access pthread_mutex_t) return int;
   --  pragma Inline (pthread_cond_wait); <-- MaRTE OS

   function pthread_cond_timedwait
     (cond    : access pthread_cond_t;
      mutex   : access pthread_mutex_t;
      abstime : access timespec) return int;
   --  pragma Inline (pthread_cond_timedwait); <-- MaRTE OS

   --------------------------
   -- POSIX.1c  Section 13 --
   --------------------------

   --  << MaRTE OS
   PTHREAD_PRIO_PROTECT : constant :=
     General_Constants.HIGHEST_CEILING_PRIORITY;

   function pthread_mutexattr_setprotocol
     (attr     : access pthread_mutexattr_t;
      protocol : int) return int
     renames Pthread.Pthread_Mutexattr_Setprotocol;

   function pthread_mutexattr_setprioceiling
     (attr        : access pthread_mutexattr_t;
      prioceiling : int) return int
     renames Pthread.Pthread_Mutexattr_Setprioceiling;
   --  MaRTE OS >>

   type struct_sched_param is record
      sched_priority : int;  --  scheduling priority
   end record;
   pragma Convention (C, struct_sched_param);

   function pthread_setschedparam
     (thread : pthread_t;
      policy : int;
      param  : access struct_sched_param) return int;
   --  pragma Import (C, pthread_setschedparam,
   --                 "pthread_setschedparam"); <-- MaRTE OS

   function pthread_attr_setschedpolicy
     (attr   : access pthread_attr_t;
      policy : int) return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Setschedpolicy;
   --  pragma Import (C, pthread_attr_setschedpolicy,
   --                 "pthread_attr_setschedpolicy"); <-- MaRTE OS

   function sched_yield return int renames Sched.Sched_Yield;
   --  pragma Import (C, sched_yield, "sched_yield"); <-- MaRTE OS

   ---------------------------
   -- P1003.1c - Section 16 --
   ---------------------------

   function pthread_attr_init
     (attributes : access pthread_attr_t) return int
     renames Pthread.Pthread_Attr_Init;
   --  pragma Import (C, pthread_attr_init, "pthread_attr_init"); <-- MaRTE OS

   function pthread_attr_destroy
     (attributes : access pthread_attr_t) return int
     renames Pthread.Pthread_Attr_Destroy;
   --  pragma Import (C, pthread_attr_destroy,
   --               "pthread_attr_destroy"); <-- MaRTE OS

   function pthread_attr_setdetachstate
     (attr        : access pthread_attr_t;
      detachstate : int) return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Setdetachstate;
   --  pragma Import (C, pthread_attr_setdetachstate,
   --                 "pthread_attr_setdetachstate"); <-- MaRTE OS

   function pthread_attr_setstacksize
     (attr      : access pthread_attr_t;
      stacksize : size_t) return int;
   --  pragma Import (C, pthread_attr_setstacksize,
   --               "pthread_attr_setstacksize"); <-- MaRTE OS

   function pthread_create
     (thread        : access pthread_t;
      attributes    : access pthread_attr_t;
      start_routine : Thread_Body;
      arg           : System.Address) return int;
   --  pragma Import (C, pthread_create, "pthread_create"); <-- MaRTE OS

   procedure pthread_exit (status : System.Address)
     renames Pthread.Pthread_Exit;
   --  pragma Import (C, pthread_exit, "pthread_exit"); <-- MaRTE OS

   function pthread_self return pthread_t;
   --  pragma Import (C, pthread_self, "pthread_self"); <-- MaRTE OS

   --------------------------
   -- POSIX.1c  Section 17 --
   --------------------------

   function pthread_setspecific
     (key   : pthread_key_t;
      value : System.Address) return int
     renames Pthread.Pthread_Setspecific;
   --  pragma Import (C, pthread_setspecific,
   --                 "pthread_setspecific"); <-- MaRTE OS

   function pthread_getspecific (key : pthread_key_t) return System.Address
     renames Pthread.Pthread_Getspecific;
   --  pragma Import (C, pthread_getspecific,
   --               "pthread_getspecific"); <-- MaRTE OS

   type destructor_pointer is access procedure (arg : System.Address);

   function pthread_key_create
     (key        : access pthread_key_t;
      destructor : destructor_pointer) return int;
   --  pragma Import (C, pthread_key_create,
   --               "pthread_key_create"); <-- MaRTE OS

private

   --  << MaRTE OS
--     type sigset_t is array (0 .. 127) of unsigned_char;
--     pragma Convention (C, sigset_t);

--     type pid_t is new int;

--     type time_t is new long;

--     type timespec is record
--        tv_sec  : time_t;
--        tv_nsec : long;
--     end record;
--     pragma Convention (C, timespec);

--     type struct_timeval is record
--        tv_sec  : time_t;
--        tv_usec : time_t;
--     end record;
--     pragma Convention (C, struct_timeval);

--     type pthread_attr_t is record
--        detachstate   : int;
--        schedpolicy   : int;
--        schedparam    : struct_sched_param;
--        inheritsched  : int;
--        scope         : int;
--        guardsize     : size_t;
--        stackaddr_set : int;
--        stackaddr     : System.Address;
--        stacksize     : size_t;
--     end record;
--     pragma Convention (C, pthread_attr_t);

--     type pthread_condattr_t is record
--        dummy : int;
--     end record;
--     pragma Convention (C, pthread_condattr_t);

--     type pthread_mutexattr_t is record
--        mutexkind : int;
--     end record;
--     pragma Convention (C, pthread_mutexattr_t);

--     type struct_pthread_fast_lock is record
--        status   : long;
--        spinlock : int;
--     end record;
--     pragma Convention (C, struct_pthread_fast_lock);

--     type pthread_mutex_t is record
--        m_reserved : int;
--        m_count    : int;
--        m_owner    : System.Address;
--        m_kind     : int;
--        m_lock     : struct_pthread_fast_lock;
--     end record;
--     pragma Convention (C, pthread_mutex_t);

--     type pthread_cond_t is array (0 .. 47) of unsigned_char;
--     pragma Convention (C, pthread_cond_t);

--     type pthread_key_t is new unsigned;

   type sigset_t  is new PSignal.Signal_Set;

   type pthread_mutex_t     is new Pthread.Pthread_Mutex_T;
   --  MaRTE OS >>

end System.OS_Interface;
