with Ada.Application_Scheduling;
with MaRTE.Kernel;
with MaRTE.Kernel.Mutexes.Attributes_SRP;
with MaRTE.Kernel.Tasks_Operations.Attributes_SRP;

package System.OS_Interface.Application_Scheduling is

   package K renames MaRTE.Kernel;

   ---------------------------------------------------------------------------
   --  SRP support  --------------------------------------------------------
   ---------------------------------------------------------------------------

   subtype preemption_level_t is K.Task_Preemption_Level;
   --  Numeric unsigned value with 16 bits

   --  Preemption Level in mutex attributes
   function Pthread_Mutexattr_Setpreemptionlevel
     (Attr  : access pthread_mutexattr_t;
      Level : in     preemption_level_t)
     return int
     renames
     K.Mutexes.Attributes_SRP.Pthread_Mutexattr_Setpreemptionlevel;

   function Pthread_Mutexattr_Getpreemptionlevel
     (Attr  : access pthread_mutexattr_t;
      Level : access preemption_level_t)
     return int
     renames
     K.Mutexes.Attributes_SRP.Pthread_Mutexattr_Getpreemptionlevel;

   --  Preemption Level in task attributes
   function Pthread_Attr_Setpreemptionlevel
     (Attr  : access pthread_attr_t;
      Level : in     preemption_level_t)
     return int
     renames
     K.Tasks_Operations.Attributes_SRP.Pthread_Attr_Setpreemptionlevel;

   function Pthread_Attr_Getpreemptionlevel
     (Attr  : access pthread_attr_t;
      Level : access preemption_level_t)
     return int
     renames
     K.Tasks_Operations.Attributes_SRP.Pthread_Attr_Getpreemptionlevel;

   ----------------------------------------------------------------------------
   --  Set and Get Application Defined Sched Parameters  ----------------------
   ----------------------------------------------------------------------------
   type Scheduling_Parameters_Ac is
     access all Ada.Application_Scheduling.Scheduling_Parameters'Class;
   subtype Scheduling_Parameters is
     Ada.Application_Scheduling.Scheduling_Parameters;

   subtype Scheduling_Parameters_Size is K.AppSched_Param_Size_T;

   function Pthread_Attr_Setappschedparam
     (Attr                : access pthread_attr_t;
      AppSched_Param      : in     Scheduling_Parameters_Ac;
      AppSched_Param_Size : in     Scheduling_Parameters_Size)
     return int;

   function Pthread_Setappschedparam
     (T                   : in     pthread_t;
      AppSched_Param      : access Scheduling_Parameters'Class;
      AppSched_Param_Size : in     Scheduling_Parameters_Size)
     return int;

   function Pthread_Getappschedparam
     (T                   : in     pthread_t;
      AppSched_Param      : in     Scheduling_Parameters_Ac;
      AppSched_Param_Size : access Scheduling_Parameters_Size)
     return int;

end System.OS_Interface.Application_Scheduling;
