------------------------------------------------------------------------------
--                            TEST of IDE module                            --
------------------------------------------------------------------------------
--  Test for the ide and ide.partitions files. It also works as a model for --
--  understanding how to use them.                                          --
--  The best way to test the module is to use a PC emulator and check the   --
--  results with Linux tools.                                               --
--  i.e.:  dd if=mydisk.img bs=512 count=1 skip=63 | od -t x1               --
--  i.e.:  qemu -fda grubfloppy.img -hda mydisk.img -boot a                 --
--                                            daniel.sangorrin_at_gmail.com --
------------------------------------------------------------------------------
--  TODO: buffer with more than one sector
--  TODO: check Return values for posible errors
------------------------------------------------------------------------------
with MaRTE_OS;
--  Basic types
with MaRTE.Integer_Types; use MaRTE.Integer_Types;
--  Text input/output
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;
with Ada.Text_IO; use Ada.Text_IO;
--  For creating the buffer
with Ada.Unchecked_Conversion, System;
--  IDE interface
with IDE;
with IDE.Partitions;

procedure test_ide is
   --    procedure Pause is
   --       H : Character;
   --    begin
   --       Put("Debug: Pause...");
   --       Get_Immediate(H);
   --       New_Line;
   --    end Pause;
   use type IDE.Sector_Count;
   function To_Buffer_Ac is new Ada.Unchecked_Conversion (System.Address, IDE.FSDT.Buffer_Ac);
   Drv : IDE.Drive := IDE.hda;
   Start : IDE.Sector_Offset := 0;
   Count : IDE.Sector_Count  := 3;
   Sect : IDE.Sector;
   Buffer : IDE.FSDT.Buffer_Ac;
   Part : aliased IDE.Partitions.Partition;
   Ret : Int;
begin
   --  Init the disk
   Ret := IDE.Init (Drv);

   --  Get the information of the 1st partition from the MBR
   Ret := IDE.Partitions.Get_Partition_Info (Drv, 1, Part'Access);
   Put_Line ("State: "&IDE.Partitions.Partition_State'Image (Part.Stat));
   Put_Line ("Type: "&IDE.Partitions.Partition_Type'Image (Part.P_Type));
   Put_Line ("First sector: "&IDE.Sector_Offset'Image (Part.First_Sector));
   Put_Line ("Size: "&IDE.Sector_Offset'Image (Part.Size));

   --  Go to that partition and read the first sector (put some bytes only)
   Buffer := To_Buffer_Ac (Sect'Address);
   Ret := IDE.RW_Sectors (Drv, IDE.Read, Part.First_Sector, 1, Buffer);
   for j in IDE.FSDT.Buffer_Length range 1 .. 10 loop
      Put (Integer (Buffer (j)), 5, 16); Put(" ");
   end loop;
   New_Line;

   --  Write some bytes in the next sector
   Sect (0) := 16#67#;
   Sect (1) := 16#68#;
   Sect (2) := 16#69#;
   Sect (3) := 16#70#;
   Ret := IDE.RW_Sectors (Drv, IDE.Write, Part.First_Sector + 1, 1, Buffer);

   --  Read that sector again
   Ret := IDE.RW_Sectors (Drv, IDE.Read, Part.First_Sector + 1, 1, Buffer);
   for j in IDE.FSDT.Buffer_Length range 1 .. 4 loop
      Put (Integer (Buffer (j)), 5, 16); Put(" ");
   end loop;

   --  Free the disk
   Ret := IDE.Free (Drv);

end test_ide;